import pyetcd
import etcdgrpc

class EtcdClient(object):
    '''
    The Util which implements KV and txn service
    '''
    def __init__(self, endpoints):
        '''
        The format of endpoints
        IP_1:Port1,IP_2:Port2...
        '''
        super().__init__()
        self.balancerSrv = pyetcd.Balancer(endpoints)
        self.kvSrv = pyetcd.KV(self.balancerSrv)

    def Put(self, key, value):
        """Save a key with a value\n
        When everyting is fine, it would return OK
        """
        if (key != ""):
            return self.kvSrv.Put(key,value)
        
        raise ValueError("Key could not be empty!")

    def Get(self, key):
        if (key != ""):
            return self.kvSrv.Get(key)
            
        raise ValueError("Key could not be empty")
        

    def Del(self, key):
        key = key.strip()
        if (key != ""):
            return self.kvSrv.Del(key)
        
        raise ValueError("Key could not be empty")





import etcdgrpc

class KV(object):
    NO_AVAILABLE_CHANNEL = "No channels availabel. Please check your cluster or your arguments..."
    def __init__(self, balancer):
        super().__init__()
        self._balancer = balancer

    def Put(self, key, value):
        channel = self._balancer.AvailableChannel()

        if (channel == None):
            return self.NO_AVAILABLE_CHANNEL

        stub = etcdgrpc.KVStub(channel)
        req = etcdgrpc.PutRequest()
        req.key = key.encode('utf-8')
        req.value = value.encode('utf-8')
        response  = stub.Put(req)
        return "OK"

    def Get(self, key):
        channel = self._balancer.AvailableChannel()

        if (channel == None):
            return self.NO_AVAILABLE_CHANNEL

        stub = etcdgrpc.KVStub(channel)
        rr = etcdgrpc.RangeRequest()
        rr.key = key.encode('utf-8')
        rrr = stub.Range(rr)
        return rrr.kvs[0].value.decode('utf-8')

    def Del(self, key):
        channel = self._balancer.AvailableChannel()

        if (channel == None):
            return self.NO_AVAILABLE_CHANNEL
        
        stub = etcdgrpc.KVStub(channel)
        drr = etcdgrpc.DeleteRangeRequest()
        drr.key = key.encode('utf-8')
        response = stub.DeleteRange(drr)
        return response.deleted == 1



import grpc

class Balancer(object):
    endPoints = []
    pool = []
    
    def __init__(self, endpoints):
        super().__init__()
        self.endPoints = str.split(endpoints,',')
        self.CreatePool(self.endPoints)


    def CreatePool(self, endPoints):
        for ep in endPoints:
            channel = grpc.insecure_channel(ep)
            self.pool.append(channel)
        pass

    '''
    Make it work first..
    '''
    def AvailableChannel(self):
        if (len(self.pool) > 0):
            return self.pool[0]
        
        return None


import setuptools

with open("README.md","r") as fh:
    long_description = fh.read()


setuptools.setup(
    name = "pyetcd3",
    version = "1.0.0",
    author = "Alias",
    author_email = "aliasmic@live.cn",
    description = "Python etcd v3 client library",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://github.com/caslt/pyetcd3",
    packages = setuptools.find_packages(),
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 2 - Pre-Alpha",
    ],
    python_requires = '>=3.6'

)


import pytest
import pyetcd


client = pyetcd.EtcdClient('192.168.56.112:2379,192.168.56.114:2379,192.168.56.116:2379')

def test_Put_test_negative():
    assert 'foo'.upper() == 'FOO'

def test_Put_test():
    result = client.Put('xx','yy')
    assert result == 'OK'

def test_Get_negative():
    result = client.Get('xxx')
    assert result == 'yy'

def test_get():
    result = client.Get('xx')
    assert result == 'yy'

def test_Del_test_negative():
    assert 'foo'.upper() == 'FOO'

def test_Del_test():
    put_result = client.Put('x1','yy')
    del_result = client.Del('x1')
    assert del_result==True
